using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Core;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.Custom;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.NinjaScript.DrawingTools;

namespace NinjaTrader.NinjaScript.Indicators
{
	public class ABCD : Indicator
	{
		

		private Brush textColor = Brushes.Blue;

		private double valorDesviacion = 0.1;

		private int ultimoMinimo;

		private int ultimoMaximo;

		private int[] v = new int[5]; // Modificar numero de puntos +1
		
		private double ABFibo = 0.618;

		private double Error = 0.2;

		private double cDMax = 1.618;

		private Series<double> listadopuntos;

		private double desplazamiento = 2.0;

		private bool listo;

		

		protected override void OnStateChange()
		{
		
			
			if (State == State.SetDefaults)
			{
				Description = "Indicador";
				Name = "ABCD";
				Calculate = Calculate.OnEachTick;
				IsOverlay = false;
				DisplayInDataBox = true;
				DrawOnPricePanel = true;
				DrawHorizontalGridLines = true;
				DrawVerticalGridLines = true;
				PaintPriceMarkers = false;
				ScaleJustification = ScaleJustification.Right; 
				IsSuspendedWhileInactive = true;
				AddPlot(new Stroke(Brushes.RoyalBlue, 2f), (PlotStyle)0, "Acbdsenal");
				AddLine(Brushes.Gray, 0.0, NinjaTrader.Custom.Resource.NinjaScriptIndicatorZeroLine);
				Maximobarras = 0;
				ColorPatron = Brushes.Green;
				Opacidad = 10;
				ObjetivoFibo = 61.8;
				
			}


			else if (State == State.DataLoaded)
			{
				listadopuntos = new Series<double>((NinjaScriptBase)(object)this, (MaximumBarsLookBack)1);
			}
		}

		private bool maximoyminimos()
		{
			int maximo = -1;
			int minimo = -1;
			bool result = false;

			int lookBackPeriod = Math.Min(CurrentBar - ultimoMinimo, CurrentBar - ultimoMaximo);
			maximo = ZigZag(DeviationType.Percent, valorDesviacion, true).HighBar(0, 1, lookBackPeriod);
			minimo = ZigZag(DeviationType.Percent, valorDesviacion, true).LowBar(0, 1, lookBackPeriod);
			if (maximo > -1)
			{
				if (ultimoMaximo > ultimoMinimo)
				{
					
					listadopuntos.Reset(CurrentBar - ultimoMaximo);

				}
				
				ultimoMaximo = CurrentBar - maximo;

				listadopuntos[maximo] = High[maximo];
				result = true;

			}
			if (minimo > -1)
			{
				if (ultimoMinimo > ultimoMaximo)
				{
					
					listadopuntos.Reset(CurrentBar - ultimoMinimo);

				}
			
				ultimoMinimo = CurrentBar - minimo;

				listadopuntos[minimo] = Low[minimo];
				result = true;

			}
			return result;
		}

		private void creacionpuntos()
		{
			
			int puntos = 4;//Modificar  numero de puntos 
			
			for (int barra = CurrentBar; barra > 0; barra--)
			{
				if (listadopuntos.IsValidDataPointAt(barra))
				{
					
					v[puntos] = CurrentBar - barra;
					puntos--;
					if (puntos < 0)
					{
						break;
					}
				}
			}
		}


		//PATRON ABCD
	

		private bool detectarPatronAbcd()
		{

			//Logica del patron modificar
		
			bool detectado = false;

			//Definicion de las Partes
			double AB = (High[v[1]]) - (Low[v[2]]); // RANGO AB  
			double BC = (High[v[3]]) - (Low[v[2]]); //RANGO CB
			double CB = (High[v[3]]) - (Low[v[4]]); //RANGO CB


			//Comprobaciones
			bool orden = (High[v[1]]) > (Low[v[2]]) && (High[v[3]]) > (Low[v[2]]) && (High[v[3]]) > (Low[v[4]]);
			bool segemento1 = (AB * (ABFibo - Error) <= BC && AB * (ABFibo + Error) >= BC);
			
			bool segemento2 = (BC * (cDMax - Error) <= CB && BC * (cDMax + Error) >= CB);
			bool segemento3 = (AB * (1 - Error) <= CB && AB * (1 + Error) >= CB);

			

			bool total = orden && segemento1 && segemento2;

		

			if ((Maximobarras == 0 || v[1] <= Maximobarras) && total)
			{
				detectado = true;

				dibujarPatronAbcd();

			}


			return detectado;
			


		}


		private void dibujarPatronAbcd()
		{

			string text = (CurrentBar - v[1]).ToString();
			Draw.Text((NinjaScriptBase)(object)this, "1tu" + text, "A", v[1], High[v[1]] + (TickSize) * desplazamiento, textColor);
			Draw.Text((NinjaScriptBase)(object)this, "2tu" + text, "B", v[2], Low[v[2]] - (TickSize) * desplazamiento, textColor);
			Draw.Text((NinjaScriptBase)(object)this, "3tu" + text, "C", v[3], High[v[3]] + (TickSize) * desplazamiento, textColor);
			Draw.Text((NinjaScriptBase)(object)this, "4tu" + text, "D", v[4], Low[v[4]] - (TickSize) * desplazamiento, textColor);

			Draw.Triangle((NinjaScriptBase)(object)this, "primertriangulo" + text, true, v[1], High[v[1]], v[2], Low[v[2]], v[3], High[v[3]], ColorPatron, ColorPatron, Opacidad);
			Draw.Triangle((NinjaScriptBase)(object)this, "segundotriangulo" + text, true, v[3], High[v[3]], v[2], Low[v[2]], v[4], Low[v[4]], ColorPatron, ColorPatron, Opacidad);
			Signal[0] = 1.0;


		}


		protected override void OnBarUpdate()
		{
			
			if (CurrentBar < 2)
			{
				return;
			}
		
			Signal[0] = 0.0;
			if (!listo && ZigZag(DeviationType.Percent, valorDesviacion, true).HighBar(0, 2, 512) != -1 && ZigZag(DeviationType.Percent, valorDesviacion, true).LowBar(0, 2, 512) != -1)
			{
				listo = true;
			}
			
			if((IsFirstTickOfBar) && maximoyminimos())
			{
				creacionpuntos();
				detectarPatronAbcd();	
			}
			
		}


		[XmlIgnore]
		[Browsable(false)]
		public Series<double> Signal
		{
			get
			{
				
				return Values[0]; 
			}
		}


	

		[XmlIgnore]
		[Display(Name = "ColorPatron", Description = "Color del patron", Order = 1, GroupName = "Visual")]
		public Brush ColorPatron { get; set; }

		[Browsable(false)]
		public string BearishColorSerializable
		{
			get
			{
				return Serialize.BrushToString(ColorPatron);
			}
			set
			{
				ColorPatron = Serialize.StringToBrush(value);
			}
		}

	

		[Range(0, 100)]
		[Display(Name = "Opacidad", Description = "Opacidad", Order = 1, GroupName = "Visual")]
		public int Opacidad { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "ObjetivoFibo", Description = "Dibuja el objetivo de fibonacci", Order = 2, GroupName = "Visual")]
        public double ObjetivoFibo { get; set; }



        [XmlIgnore]
		[Display(Name = "Color de texto", Description = "Color del texto", Order = 3, GroupName = "Visual")]
		public Brush TextColor
		{
			get
			{
				return textColor;
			}
			set
			{
				textColor = value;
			}
		}

		[Browsable(false)]
		public string TextColorSerializable
		{
			get
			{
				return Serialize.BrushToString(TextColor);
			}
			set
			{
				TextColor = Serialize.StringToBrush(value);
			}
		}

	

		[Display(Name = "valorDesviacion", Description = "Para Ajustar el tama�o del patron", Order = 1, GroupName = "Parameters")]
		[NinjaScriptProperty]
		public double DeviationValue
		{
			get
			{
				return valorDesviacion;
			}
			set
			{
				valorDesviacion = Math.Max(0.0, value);
			}
		}

		[NinjaScriptProperty]
		[Display(Name = "XABFibo", Description = "Valor X A por defecto 0.618", Order = 2, GroupName = "Parameters")]
		public double XABFibo
		{
			get
			{
				return ABFibo;
			}
			set
			{
				ABFibo = Math.Max(0.0, value);
			}
		}

		[NinjaScriptProperty]
		[Display(Name = "Error", Description = "Erro de cada medida", Order = 3, GroupName = "Parameters")]
		public double XABAccuracy
		{
			get
			{
				return Error;
			}
			set
			{
				Error = Math.Max(0.0, value);
			}
		}

	

		[Display(Name = "CDMax", Description = "Valor por defecto 1.618 ", Order = 4, GroupName = "Parameters")]
		[NinjaScriptProperty]
		public double CDMax
		{
			get
			{
				return cDMax;
			}
			set
			{
				cDMax = Math.Max(0.0, value);
			}
		}


		[NinjaScriptProperty]
		[Range(0, int.MaxValue)]
		[Display(Name = "Maximobarras", Description = "Numero maxio de barras del patron", Order = 5, GroupName = "Parameters")]
		public int Maximobarras { get; set; }



	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private ABCD[] cacheABCD;
		public ABCD ABCD(double objetivoFibo, double deviationValue, double xABFibo, double xABAccuracy, double cDMax, int maximobarras)
		{
			return ABCD(Input, objetivoFibo, deviationValue, xABFibo, xABAccuracy, cDMax, maximobarras);
		}

		public ABCD ABCD(ISeries<double> input, double objetivoFibo, double deviationValue, double xABFibo, double xABAccuracy, double cDMax, int maximobarras)
		{
			if (cacheABCD != null)
				for (int idx = 0; idx < cacheABCD.Length; idx++)
					if (cacheABCD[idx] != null && cacheABCD[idx].ObjetivoFibo == objetivoFibo && cacheABCD[idx].DeviationValue == deviationValue && cacheABCD[idx].XABFibo == xABFibo && cacheABCD[idx].XABAccuracy == xABAccuracy && cacheABCD[idx].CDMax == cDMax && cacheABCD[idx].Maximobarras == maximobarras && cacheABCD[idx].EqualsInput(input))
						return cacheABCD[idx];
			return CacheIndicator<ABCD>(new ABCD(){ ObjetivoFibo = objetivoFibo, DeviationValue = deviationValue, XABFibo = xABFibo, XABAccuracy = xABAccuracy, CDMax = cDMax, Maximobarras = maximobarras }, input, ref cacheABCD);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.ABCD ABCD(double objetivoFibo, double deviationValue, double xABFibo, double xABAccuracy, double cDMax, int maximobarras)
		{
			return indicator.ABCD(Input, objetivoFibo, deviationValue, xABFibo, xABAccuracy, cDMax, maximobarras);
		}

		public Indicators.ABCD ABCD(ISeries<double> input , double objetivoFibo, double deviationValue, double xABFibo, double xABAccuracy, double cDMax, int maximobarras)
		{
			return indicator.ABCD(input, objetivoFibo, deviationValue, xABFibo, xABAccuracy, cDMax, maximobarras);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.ABCD ABCD(double objetivoFibo, double deviationValue, double xABFibo, double xABAccuracy, double cDMax, int maximobarras)
		{
			return indicator.ABCD(Input, objetivoFibo, deviationValue, xABFibo, xABAccuracy, cDMax, maximobarras);
		}

		public Indicators.ABCD ABCD(ISeries<double> input , double objetivoFibo, double deviationValue, double xABFibo, double xABAccuracy, double cDMax, int maximobarras)
		{
			return indicator.ABCD(input, objetivoFibo, deviationValue, xABFibo, xABAccuracy, cDMax, maximobarras);
		}
	}
}

#endregion
